<?php

namespace Drupal\group_foldershare\Plugin\Group\Relation;

use Drupal\group\Entity\GroupInterface;
use Drupal\group\Plugin\Group\Relation\GroupRelationBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\group\Entity\GroupRelationshipType;

/**
 * Provides a content enabler for nodes.
 *
 * @GroupRelationType(
 *   id = "group_foldershare",
 *   label = @Translation("Group Foldershare"),
 *   description = @Translation("Adds Foldershare integration for groups."),
 *   entity_type_id = "foldershare",
 *   entity_access = FALSE
 * )
 */


class GroupFoldershare extends GroupRelationBase {
    //   *   deriver = "Drupal\gnode\Plugin\Group\Relation\GroupNodeDeriver",
//   *   handlers = {
//  *     "access" = "Drupal\group\Plugin\GroupContentAccessControlHandler",
//  *     "permission_provider" = "Drupal\group\Plugin\GroupContentPermissionProvider",
//  *   }
    /**
     * {@inheritdoc}
     */
    public function getGroupOperations(GroupInterface $group) {
        return [];
    }

    /**
     * {@inheritdoc}
     */
    public function defaultConfiguration() {
        $config = parent::defaultConfiguration();
        $config['entity_cardinality'] = 1;
        $config['group_cardinality'] = 1;

        $config['create_group_menu_item'] = TRUE;
        $config['menu_item_label'] = 'Data';
        $config['menu_item_weight'] = 1;
        return $config;
    }

    /**
     * Get the plugin key for the installed group_content_menu plugin
     * @return int|string|null
     */
    private function get_group_content_menu_plugin_key() {
        $group_type = $this->getGroupType();
        foreach (array_keys($group_type->getInstalledPlugins()->getConfiguration()) as $plugin_key) {
            if (substr($plugin_key, 0, strlen('group_content_menu:')) == 'group_content_menu:') {
                return $plugin_key;
            }
        }
        return null;
    }

    /**
     * Get the GroupRelationshipType associated with the input menu plugin id
     * @param $plugin_id
     * @return \Drupal\group\Entity\GroupRelationshipTypeInterface|null
     */
    private function getGroupContentTypeForMenuPlugin($plugin_id) {
        $group_type_id = $this->getGroupTypeId();
        foreach(GroupRelationshipType::loadByPluginId($plugin_id) as $groupRelationshipType) {
            if ($groupRelationshipType->getGroupTypeId() === $group_type_id) {
                return $groupRelationshipType;
            }
        }
        return null;
    }


    /**
     * {@inheritdoc}
     */
    public function buildConfigurationForm(array $form, FormStateInterface $form_state) {

        $form = parent::buildConfigurationForm($form, $form_state);
        $configuration = $this->getConfiguration();

        // Disable the entity cardinality field as the functionality of this module
        // relies on a cardinality of 1. We don't just hide it, though, to keep a UI
        // that's consistent with other content enabler plugins.
        $info = $this->t("This field has been disabled by the plugin to guarantee the functionality that's expected of it.");
        $form['entity_cardinality']['#disabled'] = TRUE;
        $form['entity_cardinality']['#description'] .= '<br /><em>' . $info . '</em>';
        $form['group_cardinality']['#disabled'] = TRUE;
        $form['group_cardinality']['#description'] .= '<br /><em>' . $info . '</em>';

        if (!\Drupal::moduleHandler()->moduleExists('group_content_menu')) {
            $form['create_group_menu_item'] = [
                '#type' => 'checkbox',
                '#title' => 'Create group content menu entry for Foldershare',
                '#default_value' => FALSE,
                '#disabled' => TRUE,
                '#description' => $this->t('The <em>group content menu</em> module is not installed.'),
            ];
        }

        $group_type = $this->getGroupType();
        $group_content_menu_plugin_key = $this->get_group_content_menu_plugin_key();

        if (!$group_content_menu_plugin_key) {
            $group_type_url = $group_type->toUrl('content-plugins')->toString();
            $help = "<em>Group content menu</em> is disabled. Enable it at the <a href='$group_type_url' target='_blank'>Set available content</a> page for this group type.";
            $form['create_group_menu_item'] = [
                '#type' => 'checkbox',
                '#title' => 'Create group content menu entry for Foldershare',
                '#default_value' => FALSE,
                '#disabled' => TRUE,
                '#description' => $this->t($help),
            ];
            return $form;
        } else {
            // Get the plugin
            $group_content_menu_plugin_config = $group_type->getPlugin($group_content_menu_plugin_key)->getConfiguration();
            if (!$group_content_menu_plugin_config['auto_create_group_menu']) {
                $group_content_type_menu = $this->getGroupContentTypeForMenuPlugin($group_content_menu_plugin_key);
                $group_type_url = $group_content_type_menu->toUrl('edit-form')->toString();
                $help = "Enable the <em>group content menu</em> plugin's <a href='$group_type_url'>configuration</a>: <em>Automatically create a menu when a group is created</em>";
                $form['create_group_menu_item'] = [
                    '#type' => 'checkbox',
                    '#title' => 'Create group content menu entry for Foldershare',
                    '#default_value' => FALSE,
                    '#disabled' => TRUE,
                    '#description' => $this->t($help),
                ];
                return $form;
            }
        }

        $form['create_group_menu_item'] = [
            '#type' => 'checkbox',
            '#title' => 'Create group content menu entry for Foldershare',
            '#default_value' => $configuration['create_group_menu_item'],
        ];
        $form['menu_item_label'] = [
            '#type' => 'textfield',
            '#title' => 'Menu item title',
            '#default_value' => $configuration['menu_item_label'],
            '#required' => TRUE,
            '#states' => [
                //show this textfield only if the checkbox 'create_group_menu_item' is selected above
                'visible' => [
                    ':input[name="create_group_menu_item"]' => ['checked' => TRUE],
                ],
            ],
        ];
        $form['menu_item_weight'] = [
            '#type' => 'number',
            '#title' => 'Menu item weight',
            '#default_value' => $configuration['menu_item_weight'],
            '#required' => TRUE,
            '#states' => [
                //show this textfield only if the checkbox 'create_group_menu_item' is selected above
                'visible' => [
                    ':input[name="create_group_menu_item"]' => ['checked' => TRUE],
                ],
            ],
        ];
        return $form;
    }

    /**
     * {@inheritdoc}
     */
    public function calculateDependencies() {
        $dependencies = parent::calculateDependencies();
        $dependencies['module'][] = 'foldershare';
        $dependencies['config'][] = 'group_foldershare.group_foldershare_type';
        return $dependencies;
    }

    /**
     * {@inheritdoc}
     */
    public function submitConfigurationForm(array &$form, $form_state) {
        $this->configuration['create_group_menu_item'] = $form_state->getValue('create_group_menu_item');
        $this->configuration['menu_item_label'] = $form_state->getValue('menu_item_label');
        $this->configuration['menu_item_weight'] = $form_state->getValue('menu_item_weight');
    }
}
